%-------------------------------------------------------------------------------------------------
% Graph Diameter
% Author : Sai Akhil P
% Input  : Edge List of a directed graph with nodes labelled by numbers
% Output : Diameter of graph (in terms of number of hops)
% This function, dia calculates the diameter of a directed graph
%-------------------------------------------------------------------------------------------------
function diameter = dia(edgeList)

% Validating Input
sizeofA = size(edgeList);
if sizeofA(2)~=2 && sizeofA(1) ~= 2 % Check if the input has either two rows or two columns
    error('Input must be an edge list')
end
if ~isnumeric(edgeList) % Check if input is numeric
    error('Nodes must be labelled by numbers')
end

if sizeofA(2)~=2
    edgeList = edgeList';
end

minnode = min(edgeList(:));
if minnode~=1
    edgeList = edgeList - (minnode-1); % Modify nodes labels so that the numbering starts from 1
end
match = edgeList(:,1) == edgeList(:,2);
edgeList(match,:) = []; % Removing self loops

maxnode = max(edgeList(:));
edgeList = sortrows(edgeList,1); % sort the edgeList according to source nodes
count = accumarray(edgeList(:,1),1); % count(i) gives number of edges from node i

s = ones(1,maxnode);
s(1,1:size(count,1)) = cumsum(count)-count+1;
s(count==0) = 1; 
% s(i) gives the index of start of node i in sorted edgeList
% nodes not present in source nodes list are given a starting index of 1

e = zeros(1,maxnode);
e(1,1:size(count,1)) = cumsum(count);
e(count==0) = 0;
% e(i) gives the index of end of node i in sorted edgeList
% nodes not present in source nodes list are given a ending index of 0

onesarr = -1*ones(1,maxnode);
zeroarr = zeros(1,maxnode);
node = unique(edgeList(:,1))'; % Nodes for which dijkstra's single source shortest path algorithm is applied
edgeList(:,1)=[];
edgeList = edgeList';
diaofg = 0; % set diameter of graph to zero

% Running dijkstra's single source shortest path algorithm on every node
for j = node
    i = 0; % Initiating number of hops
    tf = 1;
    dij = onesarr;
    dij(j) = 1; % Nodes reached by source node j are marked '1' and node that are not yet reached are marked '-1'
    kk = j;
    while tf
        ar = zeroarr;
        
        % Finding nodes reachable by nodes in 'kk'
        start = s(kk);
        stop = e(kk);
        len = stop - start + 1;
        pos = len > 0;
        start = start(pos);
        stop = stop(pos);
        len = len(pos);
        if isempty(len) % if no nodes are reachable by nodes in 'kk' then dijkstra is ended
            tf = 0;
            if i > diaofg % if max number of hops from node j is more than diaofg
                diaofg = i; % new diaofg is equal to i
            end
        else
            endlocs = cumsum(len);
            incr = ones(1, endlocs(end));
            jumps = start(2:end) - stop(1:end-1);
            incr(endlocs(1:end-1)+1) = jumps;
            incr(1) = start(1);
            v = cumsum(incr);
            ar(edgeList(v)) = 1; % nodes reached by nodes in 'kk' are marked 1
            k = (ar+dij)==0; % nodes reachable by 'kk' which are not yet marked 1 are given by 'k'
            if sum(k)==0 % dijkstra ends when no more new nodes are reachable
                tf = 0;
                if i > diaofg % if max number of hops from node j is more than diaofg
                    diaofg = i; % new diaofg is equal to i
                end
            else
                kk = find(k);
                i = i+1; % number of hops
                dij(kk) = 1;
            end
        end
    end
end
diameter = diaofg; % Diameter of graph
end